# Generated by the protocol buffer compiler.  DO NOT EDIT!

from google.protobuf import descriptor
from google.protobuf import message
from google.protobuf import reflection
from google.protobuf import descriptor_pb2
# @@protoc_insertion_point(imports)



DESCRIPTOR = descriptor.FileDescriptor(
  name='Reads.proto',
  package='goby',
  serialized_pb='\n\x0bReads.proto\x12\x04goby\"0\n\x0eReadCollection\x12\x1e\n\x05reads\x18\x01 \x03(\x0b\x32\x0f.goby.ReadEntry\"\xad\x02\n\tReadEntry\x12\x12\n\nread_index\x18\x01 \x02(\r\x12\x15\n\rbarcode_index\x18\n \x01(\r\x12\x17\n\x0fread_identifier\x18\x17 \x01(\t\x12\x13\n\x0b\x64\x65scription\x18\x16 \x01(\t\x12\x13\n\x0bread_length\x18\x02 \x02(\r\x12\x10\n\x08sequence\x18\x03 \x01(\x0c\x12\x15\n\rsequence_pair\x18\x05 \x01(\x0c\x12\x18\n\x10read_length_pair\x18\x06 \x01(\r\x12\x16\n\x0equality_scores\x18\x04 \x01(\x0c\x12\x1b\n\x13quality_scores_pair\x18\x07 \x01(\x0c\x12\x17\n\x0f\x63ompressed_data\x18\x08 \x01(\x0c\x12!\n\tmeta_data\x18\x19 \x03(\x0b\x32\x0e.goby.MetaData\"&\n\x08MetaData\x12\x0b\n\x03key\x18\x01 \x02(\t\x12\r\n\x05value\x18\x02 \x02(\tB\"\n\x1e\x65\x64u.cornell.med.icb.goby.readsH\x01')




_READCOLLECTION = descriptor.Descriptor(
  name='ReadCollection',
  full_name='goby.ReadCollection',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='reads', full_name='goby.ReadCollection.reads', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=21,
  serialized_end=69,
)


_READENTRY = descriptor.Descriptor(
  name='ReadEntry',
  full_name='goby.ReadEntry',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='read_index', full_name='goby.ReadEntry.read_index', index=0,
      number=1, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='barcode_index', full_name='goby.ReadEntry.barcode_index', index=1,
      number=10, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='read_identifier', full_name='goby.ReadEntry.read_identifier', index=2,
      number=23, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='description', full_name='goby.ReadEntry.description', index=3,
      number=22, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='read_length', full_name='goby.ReadEntry.read_length', index=4,
      number=2, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='sequence', full_name='goby.ReadEntry.sequence', index=5,
      number=3, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value="",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='sequence_pair', full_name='goby.ReadEntry.sequence_pair', index=6,
      number=5, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value="",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='read_length_pair', full_name='goby.ReadEntry.read_length_pair', index=7,
      number=6, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='quality_scores', full_name='goby.ReadEntry.quality_scores', index=8,
      number=4, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value="",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='quality_scores_pair', full_name='goby.ReadEntry.quality_scores_pair', index=9,
      number=7, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value="",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='compressed_data', full_name='goby.ReadEntry.compressed_data', index=10,
      number=8, type=12, cpp_type=9, label=1,
      has_default_value=False, default_value="",
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='meta_data', full_name='goby.ReadEntry.meta_data', index=11,
      number=25, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=72,
  serialized_end=373,
)


_METADATA = descriptor.Descriptor(
  name='MetaData',
  full_name='goby.MetaData',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    descriptor.FieldDescriptor(
      name='key', full_name='goby.MetaData.key', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
    descriptor.FieldDescriptor(
      name='value', full_name='goby.MetaData.value', index=1,
      number=2, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=unicode("", "utf-8"),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      options=None),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  options=None,
  is_extendable=False,
  extension_ranges=[],
  serialized_start=375,
  serialized_end=413,
)

_READCOLLECTION.fields_by_name['reads'].message_type = _READENTRY
_READENTRY.fields_by_name['meta_data'].message_type = _METADATA
DESCRIPTOR.message_types_by_name['ReadCollection'] = _READCOLLECTION
DESCRIPTOR.message_types_by_name['ReadEntry'] = _READENTRY
DESCRIPTOR.message_types_by_name['MetaData'] = _METADATA

class ReadCollection(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _READCOLLECTION
  
  # @@protoc_insertion_point(class_scope:goby.ReadCollection)

class ReadEntry(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _READENTRY
  
  # @@protoc_insertion_point(class_scope:goby.ReadEntry)

class MetaData(message.Message):
  __metaclass__ = reflection.GeneratedProtocolMessageType
  DESCRIPTOR = _METADATA
  
  # @@protoc_insertion_point(class_scope:goby.MetaData)

# @@protoc_insertion_point(module_scope)



#
# Copyright (C) 2010 Institute for Computational Biomedicine,
#                    Weill Medical College of Cornell University
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

""" Support for reading Goby chunked "compact" files. """

import os
import struct
import StringIO
from gzip import GzipFile

def read_int(fd):
    """ Python implementation of Java's DataInputStream.readInt method.
    """
    buf = fd.read(4)
    if len(buf) == 4:
        length = struct.unpack('>I', buf)[0]
    else:
        length = 0
    return length

class MessageChunksReader(object):
    """ Class to parse a file that contains goby "chunked" data.
    The MessageChunksReader is actually an iterator over indiviual
    entries stored in the compressed file.
    """
 
    verbose = False
    """ print verbose messages """

    filename = None
    """ the name of the chunked file """

    fileobject = None
    """ handle to the actual chunked file """

    fileindex = None
    """ the current index into the chunked file """

    DELIMITER_LENGTH = 8
    """ length of the delimiter tag (in bytes) """

    def __init__(self, filename, verbose = False):
        """ Initialize the MessageChunksReader using the name
        of the of the compact file (reads, alignment entries, etc.)
        """

        self.verbose = verbose

        if self.verbose:
            print "Reading data from", filename

        # store the file information
        self.filename = filename

        # open the file
        self.fileobject = open(self.filename, "rb")

        # and set the current file pointer to the beginning of the file
        self.fileindex = 0

    def __del__(self):
        """ Close the underlying file when the object is destroyed """
        if self.verbose:
            print "closing file:", self.filename
        self.fileobject.close()

    def next(self):
        """ Return next chunk of bytes from the file. """
        try:
            #  position to point just after the next delimiter
            self.fileindex += self.DELIMITER_LENGTH
            if self.verbose:
                print "seeking to position", self.fileindex
            self.fileobject.seek(self.DELIMITER_LENGTH, os.SEEK_CUR)

            # get the number of bytes expected in the next chunk
            num_bytes = read_int(self.fileobject)
            if self.verbose:
                print "expecting", num_bytes, "in next chunk"

            # if there are no more bytes, we're done
            if (num_bytes == 0):
                raise StopIteration
            else:
                # each chunk is compressed
                buf = self.fileobject.read(num_bytes)
                buf = GzipFile("", "rb", 0, StringIO.StringIO(buf)).read()
                return buf
        finally:
            self.fileindex = self.fileobject.tell()

    def __iter__(self):
        """ The message chunks reader is actually an iterator over chunks
        in the compact file.  Each iteration will return a chunk of bytes raw
        bytes which can then be processed """
        return self

    def __str__(self):
        return self.filename



#
# Copyright (C) 2010 Institute for Computational Biomedicine,
#                    Weill Medical College of Cornell University
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

""" Contains classes that can parse binary sequence data
stored in the Goby "compact" format.
"""

import gzip
import Reads_pb2
from MessageChunks import MessageChunksReader

class ReadsReader(object):
    """ Reads sequence "reads" written in the Goby "compact" format.
    The ReadsReader is actually an iterator over indiviual
    read entries stored in the file.
    """

    filename = None
    """ name of the compact reads file """

    entries_reader = None
    """ reader for the read entries (interally stored in chunks) """

    entries = []
    """ Current chunk of read entries """

    current_entry_index = 0
    """ current entry index """
    
    def __init__(self, filename, verbose = False):
        """ Initialize the ReadsReader using the name
        of the compact file.  Goby compact reads have
        the extension ".compact-reads"
        """

        # store the filename
        self.filename = filename

        # open the entries
        self.entries_reader = ReadsCollectionReader(filename, verbose)

    def next(self):
        """ Return next read entry from the file.
        """

        # is it time to get the next chunk from the file?
        if not self.entries or self.current_entry_index >= len(self.entries):
            self.entries = self.entries_reader.next().reads
            self.current_entry_index = 0

            # If the entries came back empty, we're done
            if not self.entries:
                raise StopIteration

        entry = self.entries[self.current_entry_index]
        self.current_entry_index += 1
        return entry

    def __iter__(self):
        return self

    def __str__(self):
        return self.filename

class ReadsCollectionReader(MessageChunksReader):
    """ Iterator for read collections within a compact
    reads file
    """

    def __init__(self, filename, verbose = False):
        MessageChunksReader.__init__(self, filename, verbose)

    def next(self):
        """ Return next read collection from the
        compact reads file
        """
        buf = MessageChunksReader.next(self)
        collection = Reads_pb2.ReadCollection()
        collection.ParseFromString(buf)
        return collection

    def __iter__(self):
        return self

    def __str__(self):
        return self.filename



import os
import sys
try:
    import ez_setup
    ez_setup.use_setuptools()
except ImportError:
    pass

from distutils.core import setup

setup(
    name='goby',
    version='1.9.8.2',
    packages=['goby'],
    author='Campagne Lab',
    author_email='icb@med.cornell.edu',
    scripts=[
        'GobyAlignmentStats.py',
        'GobyAlignmentToText.py',
        'GobyCompactToFasta.py',
        'GobyReadsStats.py'
        ],
    url='http://goby.campagnelab.org/',
    description='Python API for reading binary data files created with the Goby next-gen data management framework.',
    license='GNU General Public License (GPL)',
    long_description=open('README.txt').read(),
    requires=[
        'google.protobuf (>=2.3)',
        'pyjavaproperties (>=0.3)',
        ],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Intended Audience :: Developers',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: GNU General Public License (GPL)',
        'Operating System :: OS Independent',
        'Topic :: Scientific/Engineering :: Bio-Informatics'
        ],
)



#!/usr/bin/env python

#
# Copyright (C) 2010 Institute for Computational Biomedicine,
#                    Weill Medical College of Cornell University
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

import getopt
import struct
import sys
import textwrap

from goby.Reads import ReadsReader

def usage():
    print "usage:", sys.argv[0], "[-h|--help] [-v|--verbose] [-f|--format <fasta|fastq>] [-o|--output <output-filename>] <filename>" 

def main():
    verbose = False
    format = "fasta"
    output = sys.stdout

    fake_quality_score = 40

    try:
        opts, args = getopt.getopt(sys.argv[1:], "f:o:hv", ["format=", "output=", "help", "verbose"])
    except getopt.GetoptError, err:
        print >> sys.stderr, str(err)
        usage()
        sys.exit(1)

    # Collect options
    for opt, arg in opts:
        if opt in ("-h", "--help"):
            usage()
            sys.exit()
        elif opt in ("-v", "--verbose"):
            verbose = True
        elif opt in ("-f", "--format"):
            format = arg
        elif opt in ("-o", "--output"):
            output = open(arg, "w")

    if format != "fasta" and format != "fastq":
        print >> sys.stderr, "Format", format, "is not supported"
        usage()
        sys.exit(3)

    if len(args) != 1:
        usage()
        sys.exit(2)

    filename = args[0]
    if verbose:
        print "Processing file =", filename

    if format == "fasta":
        new_entry_character = ">"
    else:
        new_entry_character = "@"

    reads_reader = ReadsReader(filename, verbose)
    for entry in reads_reader:
        # if the entry has a description, use it otherwise use the index
        if (entry.HasField("description")):
            description = entry.description
        else:
            description = entry.read_index
        print >> output, "%c%s" % (new_entry_character, description)
        for line in textwrap.wrap(entry.sequence, 60):
            print >> output, line

        if format == "fastq":
            print >> output, "+"
            quality_string = ""
            if entry.HasField("qualityScores"):
                for quality_score in entry.quality_scores:
                    quality_string += chr(struct.unpack("B", quality_score)[0] + 64)  # Assume Illumina encoding
                
            else:
                # fill the string with a constant quality code
                quality_string = quality_string.ljust(len(entry.sequence), chr(fake_quality_score + 64))

            for line in textwrap.wrap(quality_string, 60):
                print >> output, line
                
if __name__ == "__main__":
    main()



#
# Copyright (C) 2010 Institute for Computational Biomedicine,
#                    Weill Medical College of Cornell University
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.
#

""" Contains classes that can parse binary alignment data
stored in the Goby "compact" format.
"""

import gzip
import sys

import Alignments_pb2
from MessageChunks import MessageChunksReader

# Java properties - http://pypi.python.org/pypi/pyjavaproperties/
from pyjavaproperties import Properties

def get_basename(filename):
    """ Return the basename corresponding to the input alignment filename.
    Note that if the filename does have the extension known to be a
    compact alignment the returned value is the original filename.
    """
    for ext in [".entries", ".header", ".tmh", ".stats", ".counts", ".index"]:
        if filename.endswith(ext):
            return filename[:-len(ext)]
    return filename

class AlignmentReader(object):
    """ Reads alignments written in the Goby "compact" format.
    The AlignmentReader is actually an iterator over indiviual
    alignment entries stored in the file.
    """

    basename = None
    """ basename for this alignment """

    statistics = Properties()
    """ statistics from this alignment """

    header = Alignments_pb2.AlignmentHeader()
    """ alignment header """

    entries_reader = None
    """ reader for the alignment entries (interally stored in chunks) """

    entries = []
    """ Current chunk of alignment entries """

    current_entry_index = 0
    """ current entry index """

    def __init__(self, basename, verbose = False):
        """ Initialize the AlignmentReader using the
        basename of the alignment files.  Goby alignments
        are made up of files ending with:

        ".entries", ".header", ".tmh", ".stats", ".counts", ".index"

        The basename of this alignment is the full path to
        one of these files including everything but the
        extension.  If a basename is passed in with one of
        these extensions, it will be stripped.
        """

        # store the basename
        self.basename = get_basename(basename);

        # read the "stats" file
        stats_filename = self.basename + ".stats"
        if verbose:
            print "Reading properties from", stats_filename
        try:
            self.statistics.load(open(stats_filename))
        except IOError, err:
            # the "stats" file is optional
            print >>sys.stderr, "Could not read alignment statistics from", stats_filename
            print >>sys.stderr, str(err)
            pass

        # read the header
        header_filename = self.basename + ".header"
        if verbose:
            print "Reading header from", header_filename
        f = gzip.open(header_filename, "rb")
        self.header.ParseFromString(f.read())
        f.close()

        # open the entries
        self.entries_reader = AlignmentCollectionReader(self.basename, verbose)

    def next(self):
        """ Return next alignment entry from the entries file
        """
        # is it time to get the next chunk from the entries file?
        if not self.entries or self.current_entry_index >= len(self.entries):
            self.entries = self.entries_reader.next().alignment_entries
            self.current_entry_index = 0

            # If the entries came back empty, we're done
            if not self.entries:
                raise StopIteration

        entry = self.entries[self.current_entry_index]
        self.current_entry_index += 1
        return entry

    def __iter__(self):
        return self

    def __str__(self):
        return self.basename

class TooManyHitsReader(object):
    """ Reads "Too Many Hits information from alignments
    written in the Goby "compact" format
    """

    basename = None
    """ basename for this alignment """

    tmh = Alignments_pb2.AlignmentTooManyHits()
    """ too many hits """

    queryindex_to_numhits = dict()
    """ query index to number of hits """

    queryindex_to_depth = dict()
    """ query index to depth/length of match. """

    def __init__(self, basename, verbose = False):
        """ Initialize the TooManyHitsReader using the basename
        of the alignment files.
        """

        # store the basename
        self.basename = get_basename(basename)

        # read the "too many hits" info
        tmh_filename = self.basename + ".tmh"
        if verbose:
            print "Reading too many hits info from", tmh_filename

        try:
            f = gzip.open(tmh_filename, "rb")
            self.tmh.ParseFromString(f.read())
            f.close()
            for hit in self.tmh.hits:
                self.queryindex_to_numhits[hit.query_index] = hit.at_least_number_of_hits
                if hit.HasField("length_of_match"):
                    self.queryindex_to_depth[hit.query_index] = hit.length_of_match
        except IOError, err:
            # the "Too Many Hits" file is optional
            print >>sys.stderr, "Could not read \"Too Many Hits\" information from", tmh_filename
            print >>sys.stderr, "Assuming no queries have too many hits."
            print >>sys.stderr, str(err)
            pass

    def __str__(self):
        return self.basename

class AlignmentCollectionReader(MessageChunksReader):
    """ Iterator for alignment collections within the alignment entries."""

    def __init__(self, basename, verbose = False):
        MessageChunksReader.__init__(self, get_basename(basename) + ".entries", verbose)

    def next(self):
        """ Return next alignment collection from the entries file."""
        buf = MessageChunksReader.next(self)
        collection = Alignments_pb2.AlignmentCollection()
        collection.ParseFromString(buf)
        return collection

    def __iter__(self):
        return self

    def __str__(self):
        return self.basename
